"""
asammdf utility functions for channel conversions
"""

from copy import deepcopy
from typing import Literal, overload

from ..types import ChannelConversionType
from . import v2_v3_blocks as v3b
from . import v2_v3_constants as v3c
from . import v4_blocks as v4b
from . import v4_constants as v4c


@overload
def conversion_transfer(
    conversion: ChannelConversionType | None, version: Literal[3] = ..., copy: bool = ...
) -> v3b.ChannelConversion: ...


@overload
def conversion_transfer(
    conversion: ChannelConversionType, version: Literal[4], copy: bool = ...
) -> v4b.ChannelConversion: ...


def conversion_transfer(
    conversion: ChannelConversionType | None, version: int = 3, copy: bool = False
) -> ChannelConversionType:
    """convert between mdf4 and mdf3 channel conversions

    Parameters
    ----------
    conversion : block
        channel conversion
    version : int
        target mdf version
    copy : bool
        return a copy if the input conversion version is the same as the required version

    Returns
    -------
    conversion : block
        channel conversion for specified version

    """

    if version <= 3:
        if conversion is None:
            conversion = v3b.ChannelConversion(conversion_type=v3c.CONVERSION_TYPE_NONE)
        else:
            conversion_type = conversion["conversion_type"]
            if conversion.id == b"CC":
                if copy:
                    conversion = deepcopy(conversion)
            else:
                unit = conversion.unit.strip(" \r\n\t\0").encode("latin-1")

                if conversion_type == v4c.CONVERSION_TYPE_NON:
                    conversion = v3b.ChannelConversion(unit=unit, conversion_type=v3c.CONVERSION_TYPE_NONE)

                elif conversion_type == v4c.CONVERSION_TYPE_LIN:
                    conversion = v3b.ChannelConversion(
                        unit=unit,
                        conversion_type=v3c.CONVERSION_TYPE_LINEAR,
                        a=conversion.a,
                        b=conversion.b,
                    )

                elif conversion_type == v4c.CONVERSION_TYPE_RAT:
                    conversion = v3b.ChannelConversion(
                        unit=unit,
                        conversion_type=v3c.CONVERSION_TYPE_RAT,
                        P1=conversion.P1,
                        P2=conversion.P2,
                        P3=conversion.P3,
                        P4=conversion.P4,
                        P5=conversion.P5,
                        P6=conversion.P6,
                    )

                elif conversion_type == v4c.CONVERSION_TYPE_TAB:
                    conversion_ = {}
                    conversion_["ref_param_nr"] = conversion.val_param_nr // 2
                    for i in range(conversion.val_param_nr // 2):
                        conversion_[f"raw_{i}"] = conversion[f"raw_{i}"]
                        conversion_[f"phys_{i}"] = conversion[f"phys_{i}"]

                    conversion = v3b.ChannelConversion(
                        unit=unit,
                        conversion_type=v3c.CONVERSION_TYPE_TAB,
                        **conversion_,
                    )

                elif conversion_type == v4c.CONVERSION_TYPE_TABI:
                    conversion_ = {}
                    conversion_["ref_param_nr"] = conversion.val_param_nr // 2
                    for i in range(conversion.val_param_nr // 2):
                        conversion_[f"raw_{i}"] = conversion[f"raw_{i}"]
                        conversion_[f"phys_{i}"] = conversion[f"phys_{i}"]

                    conversion = v3b.ChannelConversion(
                        unit=unit,
                        conversion_type=v3c.CONVERSION_TYPE_TABI,
                        **conversion_,
                    )

                elif conversion_type == v4c.CONVERSION_TYPE_ALG:
                    formula = conversion.formula.replace("X", "X1")
                    conversion = v3b.ChannelConversion(
                        formula=formula,
                        unit=unit,
                        conversion_type=v3c.CONVERSION_TYPE_FORMULA,
                    )

                elif conversion_type == v4c.CONVERSION_TYPE_RTAB:
                    nr = (conversion.val_param_nr - 1) // 3
                    kargs = {
                        "ref_param_nr": nr,
                        "conversion_type": v3c.CONVERSION_TYPE_TABI,
                    }

                    for i in range(nr):
                        l_ = conversion[f"lower_{i}"]
                        u_ = conversion[f"upper_{i}"]
                        p_ = conversion[f"phys_{i}"]
                        kargs[f"raw_{i}"] = l_
                        kargs[f"raw_{i}"] = u_ - 0.000_001
                        kargs[f"phys_{i}"] = p_
                        kargs[f"phys_{i}"] = p_

                    conversion = v3b.ChannelConversion(unit=unit, **kargs)

                elif conversion_type == v4c.CONVERSION_TYPE_TABX:
                    nr = conversion.val_param_nr

                    kargs = {
                        "ref_param_nr": nr + 1,
                        "unit": unit,
                        "conversion_type": v3c.CONVERSION_TYPE_RTABX,
                    }
                    for i in range(nr):
                        kargs[f"lower_{i}"] = conversion[f"val_{i}"]
                        kargs[f"upper_{i}"] = conversion[f"val_{i}"]
                        if isinstance(
                            conversion.referenced_blocks[f"text_{i}"],
                            v4b.ChannelConversion,
                        ):
                            kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"].name.encode("latin-1")
                        else:
                            kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"]

                    new_conversion = v3b.ChannelConversion(**kargs)
                    if isinstance(
                        conversion.referenced_blocks["default_addr"],
                        v4b.ChannelConversion,
                    ):
                        default_addr = conversion.referenced_blocks["default_addr"].name.encode("latin-1")
                    else:
                        default_addr = conversion.referenced_blocks["default_addr"]
                    new_conversion.referenced_blocks["default_addr"] = default_addr

                    conversion = new_conversion

                elif conversion_type == v4c.CONVERSION_TYPE_RTABX:
                    nr = conversion.val_param_nr // 2
                    kargs = {
                        "ref_param_nr": nr + 1,
                        "unit": unit,
                        "conversion_type": v3c.CONVERSION_TYPE_RTABX,
                    }
                    for i in range(nr):
                        kargs[f"lower_{i}"] = conversion[f"lower_{i}"]
                        kargs[f"upper_{i}"] = conversion[f"upper_{i}"]
                        if isinstance(
                            conversion.referenced_blocks[f"text_{i}"],
                            v4b.ChannelConversion,
                        ):
                            kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"].name.encode("latin-1")
                        else:
                            kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"]

                    new_conversion = v3b.ChannelConversion(**kargs)
                    if isinstance(
                        conversion.referenced_blocks["default_addr"],
                        v4b.ChannelConversion,
                    ):
                        default_addr = conversion.referenced_blocks["default_addr"].name.encode("latin-1")
                    else:
                        default_addr = conversion.referenced_blocks["default_addr"]
                    new_conversion.referenced_blocks["default_addr"] = default_addr

                    conversion = new_conversion

    else:
        if not conversion or conversion.id == b"##CC":
            if copy:
                conversion = deepcopy(conversion)
        else:
            conversion_type = conversion.conversion_type
            unit = conversion.unit_field.decode("latin-1").strip(" \r\n\t\0")

            if conversion_type == v3c.CONVERSION_TYPE_NONE:
                conversion = v4b.ChannelConversion(conversion_type=v4c.CONVERSION_TYPE_NON)

            elif conversion_type == v3c.CONVERSION_TYPE_LINEAR:
                conversion = v4b.ChannelConversion(
                    conversion_type=v4c.CONVERSION_TYPE_LIN,
                    a=conversion.a,
                    b=conversion.b,
                )

            elif conversion_type == v3c.CONVERSION_TYPE_RAT:
                conversion = v4b.ChannelConversion(
                    conversion_type=v4c.CONVERSION_TYPE_RAT,
                    P1=conversion.P1,
                    P2=conversion.P2,
                    P3=conversion.P3,
                    P4=conversion.P4,
                    P5=conversion.P5,
                    P6=conversion.P6,
                )

            elif conversion_type == v3c.CONVERSION_TYPE_FORMULA:
                formula = conversion.formula
                conversion = v4b.ChannelConversion(conversion_type=v4c.CONVERSION_TYPE_ALG, formula=formula)

            elif conversion_type == v3c.CONVERSION_TYPE_TAB:
                conversion_ = {}
                conversion_["val_param_nr"] = conversion.ref_param_nr * 2
                for i in range(conversion.ref_param_nr):
                    conversion_[f"raw_{i}"] = conversion[f"raw_{i}"]
                    conversion_[f"phys_{i}"] = conversion[f"phys_{i}"]

                conversion = v4b.ChannelConversion(conversion_type=v4c.CONVERSION_TYPE_TAB, **conversion_)

            elif conversion_type == v3c.CONVERSION_TYPE_TABI:
                conversion_ = {}
                conversion_["val_param_nr"] = conversion.ref_param_nr * 2
                for i in range(conversion.ref_param_nr):
                    conversion_[f"raw_{i}"] = conversion[f"raw_{i}"]
                    conversion_[f"phys_{i}"] = conversion[f"phys_{i}"]

                conversion = v4b.ChannelConversion(conversion_type=v4c.CONVERSION_TYPE_TABI, **conversion_)

            elif conversion_type == v3c.CONVERSION_TYPE_TABX:
                nr = conversion["ref_param_nr"]
                kargs = {
                    "val_param_nr": nr,
                    "ref_param_nr": nr + 1,
                    "conversion_type": v4c.CONVERSION_TYPE_TABX,
                }
                for i in range(nr):
                    kargs[f"val_{i}"] = conversion[f"param_val_{i}"]
                    kargs[f"text_{i}"] = conversion[f"text_{i}"]

                conversion = v4b.ChannelConversion(**kargs)

            elif conversion_type == v3c.CONVERSION_TYPE_RTABX:
                nr = conversion["ref_param_nr"] - 1
                kargs = {
                    "val_param_nr": nr * 2,
                    "ref_param_nr": nr + 1,
                    "conversion_type": v4c.CONVERSION_TYPE_RTABX,
                    "default_addr": conversion.referenced_blocks["default_addr"],
                }
                for i in range(nr):
                    kargs[f"lower_{i}"] = conversion[f"lower_{i}"]
                    kargs[f"upper_{i}"] = conversion[f"upper_{i}"]
                    kargs[f"text_{i}"] = conversion.referenced_blocks[f"text_{i}"]

                conversion = v4b.ChannelConversion(**kargs)

            conversion.unit = unit

    return conversion


def inverse_conversion(conversion: ChannelConversionType | dict | None) -> v4b.ChannelConversion | None:

    if isinstance(conversion, v3b.ChannelConversion):
        conversion = conversion_transfer(conversion, version=4)

    if conversion:
        if not isinstance(conversion, dict):
            conversion = to_dict(conversion) or {}

        if "a" in conversion:
            conv = {
                "conversion_type": v4c.CONVERSION_TYPE_LIN,
                "a": 1 / conversion["a"],
                "b": conversion["b"] / conversion["a"],
            }
            conv = v4b.ChannelConversion(**conv)

        elif "P1" in conversion:
            a, b, c, d, e, f = (conversion[f"P{i}"] for i in range(1, 7))

            if e == 0 and f == 0:
                if d == 0 and a == 0:
                    conv = None
                else:
                    conv = {
                        "P1": 0,
                        "P2": e,
                        "P3": -b,
                        "P4": 0,
                        "P5": -d,
                        "P6": a,
                        "conversion_type": v4c.CONVERSION_TYPE_RAT,
                    }
                    conv = v4b.ChannelConversion(**conv)

            elif a == 0 and d == 0:

                if e == 0 and b == 0:
                    conv = None
                else:
                    conv = {
                        "P1": 0,
                        "P2": -f,
                        "P3": c,
                        "P4": 0,
                        "P5": e,
                        "P6": -b,
                        "conversion_type": v4c.CONVERSION_TYPE_RAT,
                    }
                    conv = v4b.ChannelConversion(**conv)

        else:
            conv = None

    else:
        conv = None

    return conv


def from_dict(conversion: dict[str, object]) -> v4b.ChannelConversion:
    conversion = dict(conversion)

    if not conversion:
        conversion = None

    elif "a" in conversion:
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_LIN
        conversion = v4b.ChannelConversion(**conversion)

    elif "formula" in conversion:
        conversion["formula"] = conversion["formula"]
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_ALG
        conversion = v4b.ChannelConversion(**conversion)

    elif all(key in conversion for key in [f"P{i}" for i in range(1, 7)]):
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_RAT
        conversion = v4b.ChannelConversion(**conversion)

    elif "raw_0" in conversion and "phys_0" in conversion:
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_TAB
        nr = 0
        while f"phys_{nr}" in conversion:
            nr += 1
        conversion["val_param_nr"] = nr * 2
        if conversion.get("interpolation", False):
            conversion["conversion_type"] = v4c.CONVERSION_TYPE_TABI
        else:
            conversion["conversion_type"] = v4c.CONVERSION_TYPE_TAB
        conversion = v4b.ChannelConversion(**conversion)

    elif "mask_0" in conversion and "text_0" in conversion:
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_BITFIELD
        nr = 0
        while f"text_{nr}" in conversion:
            val = conversion[f"text_{nr}"]
            if isinstance(val, (bytes, str)):
                partial_conversion = {
                    "conversion_type": v4c.CONVERSION_TYPE_RTABX,
                    f"upper_{nr}": conversion[f"upper_{nr}"],
                    f"lower_{nr}": conversion[f"lower_{nr}"],
                    f"text_{nr}": (
                        conversion[f"text_{nr}"]
                        if isinstance(conversion[f"text_{nr}"], bytes)
                        else conversion[f"text_{nr}"].encode("utf-8")
                    ),
                    "default": b"",
                }
                conversion[f"text_{nr}"] = from_dict(partial_conversion)
            elif isinstance(val, dict):
                conversion[f"text_{nr}"] = from_dict(val)

            nr += 1

        conversion["ref_param_nr"] = nr
        conversion["val_param_nr"] = nr
        conversion = v4b.ChannelConversion(**conversion)

    elif "upper_0" in conversion and "phys_0" in conversion:
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_RTAB
        nr = 0
        while f"phys_{nr}" in conversion:
            nr += 1
        conversion["val_param_nr"] = nr * 3 + 1
        conversion = v4b.ChannelConversion(**conversion)

    elif "val_0" in conversion and "text_0" in conversion:
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_TABX
        nr = 0
        while f"text_{nr}" in conversion:
            val = conversion[f"text_{nr}"]
            if isinstance(val, str):
                conversion[f"text_{nr}"] = val.encode("utf-8")
            elif isinstance(val, dict):
                conversion[f"text_{nr}"] = from_dict(val)
            nr += 1

        val = conversion.get("default_addr", b"")
        if isinstance(val, str):
            conversion["default_addr"] = val.encode("utf-8")
        elif isinstance(val, dict):
            conversion["default_addr"] = from_dict(val)

        conversion["ref_param_nr"] = nr + 1
        conversion = v4b.ChannelConversion(**conversion)

    elif "upper_0" in conversion and "text_0" in conversion:
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_RTABX
        nr = 0
        while f"text_{nr}" in conversion:
            val = conversion[f"text_{nr}"]
            if isinstance(val, str):
                conversion[f"text_{nr}"] = val.encode("utf-8")
            elif isinstance(val, dict):
                conversion[f"text_{nr}"] = from_dict(val)
            nr += 1

        conversion["ref_param_nr"] = nr + 1

        val = conversion.get("default_addr", b"")
        if isinstance(val, str):
            conversion["default_addr"] = val.encode("utf-8")
        elif isinstance(val, dict):
            conversion["default_addr"] = from_dict(val)
        conversion = v4b.ChannelConversion(**conversion)

    elif "default_addr" in conversion:
        conversion["conversion_type"] = v4c.CONVERSION_TYPE_TABX
        val = conversion["default_addr"]
        if isinstance(val, str):
            conversion["default_addr"] = val.encode("utf-8")
        elif isinstance(val, dict):
            conversion["default_addr"] = from_dict(val)
        conversion["ref_param_nr"] = 1
        conversion = v4b.ChannelConversion(**conversion)

    else:
        conversion = v4b.ChannelConversion(conversion_type=v4c.CONVERSION_TYPE_NON)

    return conversion


def to_dict(conversion: ChannelConversionType) -> dict | None:
    if not conversion:
        return None

    if isinstance(conversion, v3b.ChannelConversion):
        conversion = conversion_transfer(conversion, version=4)

    conversion_type = conversion.conversion_type

    conversion_dict = {
        "name": conversion.name,
        "unit": conversion.unit,
        "comment": conversion.comment,
    }

    if conversion_type == v4c.CONVERSION_TYPE_LIN:
        conversion_dict["a"] = conversion.a
        conversion_dict["b"] = conversion.b
        conversion_dict["conversion_type"] = conversion_type

    elif conversion_type == v4c.CONVERSION_TYPE_ALG:
        conversion_dict["formula"] = conversion["formula"]
        conversion_dict["conversion_type"] = conversion_type

    elif conversion_type == v4c.CONVERSION_TYPE_RAT:
        conversion_dict.update({key: conversion[key] for key in [f"P{i}" for i in range(1, 7)]})
        conversion_dict["conversion_type"] = conversion_type

    elif conversion_type in (v4c.CONVERSION_TYPE_TAB, v4c.CONVERSION_TYPE_TABI):
        params = conversion["val_param_nr"] // 2
        conversion_dict.update({key: conversion[key] for key in [f"phys_{nr}" for nr in range(params)]})
        conversion_dict.update({key: conversion[key] for key in [f"raw_{nr}" for nr in range(params)]})
        conversion_dict["conversion_type"] = conversion_type

    elif conversion_type == v4c.CONVERSION_TYPE_RTAB:
        params = (conversion["val_param_nr"] - 1) // 3
        conversion_dict.update({key: conversion[key] for key in [f"lower_{nr}" for nr in range(params)]})
        conversion_dict.update({key: conversion[key] for key in [f"upper_{nr}" for nr in range(params)]})
        conversion_dict.update({key: conversion[key] for key in [f"phys_{nr}" for nr in range(params)]})
        conversion_dict["conversion_type"] = conversion_type
        conversion_dict["default"] = conversion.default

    elif conversion_type == v4c.CONVERSION_TYPE_TABX:
        nr = conversion.ref_param_nr - 1

        conversion_dict["conversion_type"] = conversion_type

        for key, val in conversion.referenced_blocks.items():
            if isinstance(val, str):
                conversion_dict[key] = val
            elif isinstance(val, v4b.ChannelConversion):
                conversion_dict[key] = to_dict(val)
            elif val is None:
                conversion_dict[key] = ""
            else:
                conversion_dict[key] = val.decode("utf-8", errors="replace")

        for i in range(nr):
            conversion_dict[f"val_{i}"] = conversion[f"val_{i}"]

    elif conversion_type == v4c.CONVERSION_TYPE_RTABX:
        nr = conversion.ref_param_nr - 1

        conversion_dict["conversion_type"] = conversion_type

        for key, val in conversion.referenced_blocks.items():
            if isinstance(val, str):
                conversion_dict[key] = val
            elif isinstance(val, v4b.ChannelConversion):
                conversion_dict[key] = to_dict(val)
            elif val is None:
                conversion_dict[key] = ""
            else:
                conversion_dict[key] = val.decode("utf-8", errors="replace")

        for i in range(nr):
            conversion_dict[f"upper_{i}"] = conversion[f"upper_{i}"]
            conversion_dict[f"lower_{i}"] = conversion[f"lower_{i}"]

    else:
        conversion_dict = None

    return conversion_dict



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'define_conversion_dialog.ui'
##
## Created by: Qt User Interface Compiler version 6.7.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDialog, QDoubleSpinBox,
    QGridLayout, QHBoxLayout, QLabel, QLineEdit,
    QListWidgetItem, QPlainTextEdit, QPushButton, QSizePolicy,
    QSpacerItem, QStackedWidget, QTabWidget, QVBoxLayout,
    QWidget)

from asammdf.gui.widgets.list import MinimalListWidget
from . import resource_rc

class Ui_ConversionDialog(object):
    def setupUi(self, ConversionDialog):
        if not ConversionDialog.objectName():
            ConversionDialog.setObjectName(u"ConversionDialog")
        ConversionDialog.resize(937, 501)
        ConversionDialog.setMaximumSize(QSize(16777215, 16777215))
        icon = QIcon()
        icon.addFile(u":/plus.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        ConversionDialog.setWindowIcon(icon)
        ConversionDialog.setSizeGripEnabled(True)
        self.gridLayout = QGridLayout(ConversionDialog)
        self.gridLayout.setSpacing(1)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(1, 1, 1, 1)
        self.horizontalSpacer_4 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_4, 5, 2, 1, 1)

        self.label_8 = QLabel(ConversionDialog)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)

        self.gridLayout.addWidget(self.label_8, 3, 0, 1, 1)

        self.label_7 = QLabel(ConversionDialog)
        self.label_7.setObjectName(u"label_7")

        self.gridLayout.addWidget(self.label_7, 1, 0, 1, 1)

        self.unit = QLineEdit(ConversionDialog)
        self.unit.setObjectName(u"unit")

        self.gridLayout.addWidget(self.unit, 4, 1, 1, 1)

        self.label_6 = QLabel(ConversionDialog)
        self.label_6.setObjectName(u"label_6")

        self.gridLayout.addWidget(self.label_6, 4, 0, 1, 1)

        self.apply_btn = QPushButton(ConversionDialog)
        self.apply_btn.setObjectName(u"apply_btn")

        self.gridLayout.addWidget(self.apply_btn, 5, 5, 1, 1)

        self.name = QLineEdit(ConversionDialog)
        self.name.setObjectName(u"name")

        self.gridLayout.addWidget(self.name, 1, 1, 1, 6)

        self.comment = QPlainTextEdit(ConversionDialog)
        self.comment.setObjectName(u"comment")

        self.gridLayout.addWidget(self.comment, 3, 1, 1, 6)

        self.load_original_conversion_btn = QPushButton(ConversionDialog)
        self.load_original_conversion_btn.setObjectName(u"load_original_conversion_btn")

        self.gridLayout.addWidget(self.load_original_conversion_btn, 5, 3, 1, 1)

        self.cancel_btn = QPushButton(ConversionDialog)
        self.cancel_btn.setObjectName(u"cancel_btn")

        self.gridLayout.addWidget(self.cancel_btn, 5, 6, 1, 1)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_3, 5, 1, 1, 1)

        self.tabs = QTabWidget(ConversionDialog)
        self.tabs.setObjectName(u"tabs")
        self.tab = QWidget()
        self.tab.setObjectName(u"tab")
        self.gridLayout_5 = QGridLayout(self.tab)
        self.gridLayout_5.setSpacing(1)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.gridLayout_5.setContentsMargins(1, 1, 1, 1)
        self.label_10 = QLabel(self.tab)
        self.label_10.setObjectName(u"label_10")

        self.gridLayout_5.addWidget(self.label_10, 2, 0, 1, 1)

        self.b = QDoubleSpinBox(self.tab)
        self.b.setObjectName(u"b")
        self.b.setDecimals(6)

        self.gridLayout_5.addWidget(self.b, 2, 1, 1, 1)

        self.label_11 = QLabel(self.tab)
        self.label_11.setObjectName(u"label_11")

        self.gridLayout_5.addWidget(self.label_11, 0, 0, 1, 2)

        self.label_9 = QLabel(self.tab)
        self.label_9.setObjectName(u"label_9")

        self.gridLayout_5.addWidget(self.label_9, 1, 0, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout_5.addItem(self.horizontalSpacer, 1, 2, 1, 1)

        self.a = QDoubleSpinBox(self.tab)
        self.a.setObjectName(u"a")
        self.a.setMinimumSize(QSize(100, 0))
        self.a.setDecimals(6)

        self.gridLayout_5.addWidget(self.a, 1, 1, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_5.addItem(self.verticalSpacer, 3, 0, 1, 1)

        self.tabs.addTab(self.tab, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName(u"tab_2")
        self.gridLayout_6 = QGridLayout(self.tab_2)
        self.gridLayout_6.setSpacing(1)
        self.gridLayout_6.setObjectName(u"gridLayout_6")
        self.gridLayout_6.setContentsMargins(1, 1, 1, 1)
        self.label_3 = QLabel(self.tab_2)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout_6.addWidget(self.label_3, 1, 0, 1, 1)

        self.verticalSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_6.addItem(self.verticalSpacer_2, 7, 0, 1, 1)

        self.p3 = QDoubleSpinBox(self.tab_2)
        self.p3.setObjectName(u"p3")
        self.p3.setDecimals(6)

        self.gridLayout_6.addWidget(self.p3, 3, 1, 1, 1)

        self.p1 = QDoubleSpinBox(self.tab_2)
        self.p1.setObjectName(u"p1")
        self.p1.setMinimumSize(QSize(100, 0))
        self.p1.setDecimals(6)

        self.gridLayout_6.addWidget(self.p1, 1, 1, 1, 1)

        self.p2 = QDoubleSpinBox(self.tab_2)
        self.p2.setObjectName(u"p2")
        self.p2.setDecimals(6)

        self.gridLayout_6.addWidget(self.p2, 2, 1, 1, 1)

        self.label_4 = QLabel(self.tab_2)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_6.addWidget(self.label_4, 2, 0, 1, 1)

        self.label_15 = QLabel(self.tab_2)
        self.label_15.setObjectName(u"label_15")

        self.gridLayout_6.addWidget(self.label_15, 6, 0, 1, 1)

        self.label_13 = QLabel(self.tab_2)
        self.label_13.setObjectName(u"label_13")

        self.gridLayout_6.addWidget(self.label_13, 4, 0, 1, 1)

        self.p6 = QDoubleSpinBox(self.tab_2)
        self.p6.setObjectName(u"p6")
        self.p6.setDecimals(6)

        self.gridLayout_6.addWidget(self.p6, 6, 1, 1, 1)

        self.p5 = QDoubleSpinBox(self.tab_2)
        self.p5.setObjectName(u"p5")
        self.p5.setDecimals(6)

        self.gridLayout_6.addWidget(self.p5, 5, 1, 1, 1)

        self.label_5 = QLabel(self.tab_2)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout_6.addWidget(self.label_5, 3, 0, 1, 1)

        self.p4 = QDoubleSpinBox(self.tab_2)
        self.p4.setObjectName(u"p4")
        self.p4.setDecimals(6)

        self.gridLayout_6.addWidget(self.p4, 4, 1, 1, 1)

        self.label_14 = QLabel(self.tab_2)
        self.label_14.setObjectName(u"label_14")

        self.gridLayout_6.addWidget(self.label_14, 5, 0, 1, 1)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout_6.addItem(self.horizontalSpacer_2, 1, 2, 1, 1)

        self.label_2 = QLabel(self.tab_2)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_6.addWidget(self.label_2, 0, 0, 1, 3)

        self.tabs.addTab(self.tab_2, "")
        self.tab_3 = QWidget()
        self.tab_3.setObjectName(u"tab_3")
        self.gridLayout_2 = QGridLayout(self.tab_3)
        self.gridLayout_2.setSpacing(1)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(1, 1, 1, 1)
        self.vtt_list = MinimalListWidget(self.tab_3)
        self.vtt_list.setObjectName(u"vtt_list")

        self.gridLayout_2.addWidget(self.vtt_list, 1, 0, 4, 4)

        self.vtt_mode = QStackedWidget(self.tab_3)
        self.vtt_mode.setObjectName(u"vtt_mode")
        self.page = QWidget()
        self.page.setObjectName(u"page")
        self.horizontalLayout = QHBoxLayout(self.page)
        self.horizontalLayout.setSpacing(1)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(1, 1, 1, 1)
        self.vtt_default = QLineEdit(self.page)
        self.vtt_default.setObjectName(u"vtt_default")

        self.horizontalLayout.addWidget(self.vtt_default)

        self.vtt_mode.addWidget(self.page)
        self.page_2 = QWidget()
        self.page_2.setObjectName(u"page_2")
        self.horizontalLayout_2 = QHBoxLayout(self.page_2)
        self.horizontalLayout_2.setSpacing(1)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.vtt_default_btn = QPushButton(self.page_2)
        self.vtt_default_btn.setObjectName(u"vtt_default_btn")

        self.horizontalLayout_2.addWidget(self.vtt_default_btn)

        self.vtt_mode.addWidget(self.page_2)

        self.gridLayout_2.addWidget(self.vtt_mode, 5, 2, 1, 1)

        self.verticalSpacer_3 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_2.addItem(self.verticalSpacer_3, 2, 4, 1, 1)

        self.insert_btn = QPushButton(self.tab_3)
        self.insert_btn.setObjectName(u"insert_btn")

        self.gridLayout_2.addWidget(self.insert_btn, 1, 4, 1, 1)

        self.label_16 = QLabel(self.tab_3)
        self.label_16.setObjectName(u"label_16")

        self.gridLayout_2.addWidget(self.label_16, 5, 0, 1, 1)

        self.label = QLabel(self.tab_3)
        self.label.setObjectName(u"label")

        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 4)

        self.reset_btn = QPushButton(self.tab_3)
        self.reset_btn.setObjectName(u"reset_btn")

        self.gridLayout_2.addWidget(self.reset_btn, 4, 4, 1, 1)

        self.vtt_default_mode = QComboBox(self.tab_3)
        self.vtt_default_mode.addItem("")
        self.vtt_default_mode.addItem("")
        self.vtt_default_mode.setObjectName(u"vtt_default_mode")

        self.gridLayout_2.addWidget(self.vtt_default_mode, 5, 1, 1, 1)

        self.tabs.addTab(self.tab_3, "")
        self.tab_4 = QWidget()
        self.tab_4.setObjectName(u"tab_4")
        self.gridLayout_3 = QGridLayout(self.tab_4)
        self.gridLayout_3.setSpacing(1)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setContentsMargins(1, 1, 1, 1)
        self.vrtt_list = MinimalListWidget(self.tab_4)
        self.vrtt_list.setObjectName(u"vrtt_list")

        self.gridLayout_3.addWidget(self.vrtt_list, 1, 0, 3, 4)

        self.label_12 = QLabel(self.tab_4)
        self.label_12.setObjectName(u"label_12")

        self.gridLayout_3.addWidget(self.label_12, 0, 0, 1, 4)

        self.vrtt_mode = QStackedWidget(self.tab_4)
        self.vrtt_mode.setObjectName(u"vrtt_mode")
        self.page_3 = QWidget()
        self.page_3.setObjectName(u"page_3")
        self.horizontalLayout_3 = QHBoxLayout(self.page_3)
        self.horizontalLayout_3.setSpacing(1)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(1, 1, 1, 1)
        self.vrtt_default = QLineEdit(self.page_3)
        self.vrtt_default.setObjectName(u"vrtt_default")

        self.horizontalLayout_3.addWidget(self.vrtt_default)

        self.vrtt_mode.addWidget(self.page_3)
        self.page_4 = QWidget()
        self.page_4.setObjectName(u"page_4")
        self.horizontalLayout_4 = QHBoxLayout(self.page_4)
        self.horizontalLayout_4.setSpacing(1)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalLayout_4.setContentsMargins(1, 1, 1, 1)
        self.vrtt_default_btn = QPushButton(self.page_4)
        self.vrtt_default_btn.setObjectName(u"vrtt_default_btn")

        self.horizontalLayout_4.addWidget(self.vrtt_default_btn)

        self.vrtt_mode.addWidget(self.page_4)

        self.gridLayout_3.addWidget(self.vrtt_mode, 4, 2, 1, 1)

        self.insert_vrtt_btn = QPushButton(self.tab_4)
        self.insert_vrtt_btn.setObjectName(u"insert_vrtt_btn")

        self.gridLayout_3.addWidget(self.insert_vrtt_btn, 1, 4, 1, 1)

        self.verticalSpacer_4 = QSpacerItem(20, 78, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_3.addItem(self.verticalSpacer_4, 2, 4, 1, 1)

        self.reset_vrtt_btn = QPushButton(self.tab_4)
        self.reset_vrtt_btn.setObjectName(u"reset_vrtt_btn")

        self.gridLayout_3.addWidget(self.reset_vrtt_btn, 3, 4, 1, 1)

        self.label_17 = QLabel(self.tab_4)
        self.label_17.setObjectName(u"label_17")

        self.gridLayout_3.addWidget(self.label_17, 4, 0, 1, 1)

        self.vrtt_default_mode = QComboBox(self.tab_4)
        self.vrtt_default_mode.addItem("")
        self.vrtt_default_mode.addItem("")
        self.vrtt_default_mode.setObjectName(u"vrtt_default_mode")

        self.gridLayout_3.addWidget(self.vrtt_default_mode, 4, 1, 1, 1)

        self.tabs.addTab(self.tab_4, "")
        self.tab_5 = QWidget()
        self.tab_5.setObjectName(u"tab_5")
        self.verticalLayout = QVBoxLayout(self.tab_5)
        self.verticalLayout.setSpacing(1)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(1, 1, 1, 1)
        self.label_18 = QLabel(self.tab_5)
        self.label_18.setObjectName(u"label_18")

        self.verticalLayout.addWidget(self.label_18)

        self.verticalSpacer_5 = QSpacerItem(20, 166, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer_5)

        self.tabs.addTab(self.tab_5, "")

        self.gridLayout.addWidget(self.tabs, 0, 0, 1, 7)

        self.horizontalSpacer_5 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_5, 5, 4, 1, 1)

        self.gridLayout.setRowStretch(0, 1)
        self.gridLayout.setColumnStretch(2, 1)
        QWidget.setTabOrder(self.tabs, self.a)
        QWidget.setTabOrder(self.a, self.b)
        QWidget.setTabOrder(self.b, self.p1)
        QWidget.setTabOrder(self.p1, self.p2)
        QWidget.setTabOrder(self.p2, self.p3)
        QWidget.setTabOrder(self.p3, self.p4)
        QWidget.setTabOrder(self.p4, self.p5)
        QWidget.setTabOrder(self.p5, self.p6)
        QWidget.setTabOrder(self.p6, self.vtt_list)
        QWidget.setTabOrder(self.vtt_list, self.insert_btn)
        QWidget.setTabOrder(self.insert_btn, self.reset_btn)
        QWidget.setTabOrder(self.reset_btn, self.vtt_default_mode)
        QWidget.setTabOrder(self.vtt_default_mode, self.vtt_default)
        QWidget.setTabOrder(self.vtt_default, self.vrtt_list)
        QWidget.setTabOrder(self.vrtt_list, self.insert_vrtt_btn)
        QWidget.setTabOrder(self.insert_vrtt_btn, self.reset_vrtt_btn)
        QWidget.setTabOrder(self.reset_vrtt_btn, self.vrtt_default_mode)
        QWidget.setTabOrder(self.vrtt_default_mode, self.vrtt_default)
        QWidget.setTabOrder(self.vrtt_default, self.name)
        QWidget.setTabOrder(self.name, self.comment)
        QWidget.setTabOrder(self.comment, self.unit)
        QWidget.setTabOrder(self.unit, self.apply_btn)
        QWidget.setTabOrder(self.apply_btn, self.cancel_btn)
        QWidget.setTabOrder(self.cancel_btn, self.vrtt_default_btn)
        QWidget.setTabOrder(self.vrtt_default_btn, self.vtt_default_btn)

        self.retranslateUi(ConversionDialog)

        self.tabs.setCurrentIndex(0)
        self.vtt_mode.setCurrentIndex(0)
        self.vrtt_mode.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(ConversionDialog)
    # setupUi

    def retranslateUi(self, ConversionDialog):
        ConversionDialog.setWindowTitle(QCoreApplication.translate("ConversionDialog", u"Define new channel", None))
        self.label_8.setText(QCoreApplication.translate("ConversionDialog", u"Conversion comment", None))
        self.label_7.setText(QCoreApplication.translate("ConversionDialog", u"Conversion name", None))
        self.unit.setPlaceholderText("")
        self.label_6.setText(QCoreApplication.translate("ConversionDialog", u"Conversion unit", None))
        self.apply_btn.setText(QCoreApplication.translate("ConversionDialog", u"Apply", None))
        self.name.setInputMask("")
        self.name.setText("")
        self.name.setPlaceholderText("")
        self.load_original_conversion_btn.setText(QCoreApplication.translate("ConversionDialog", u"Load original conversion", None))
        self.cancel_btn.setText(QCoreApplication.translate("ConversionDialog", u"Cancel", None))
        self.label_10.setText(QCoreApplication.translate("ConversionDialog", u"Offset (b)", None))
        self.label_11.setText(QCoreApplication.translate("ConversionDialog", u"Y = a * X + b", None))
        self.label_9.setText(QCoreApplication.translate("ConversionDialog", u"Factor (a)", None))
        self.tabs.setTabText(self.tabs.indexOf(self.tab), QCoreApplication.translate("ConversionDialog", u"Linear", None))
        self.label_3.setText(QCoreApplication.translate("ConversionDialog", u"P1", None))
        self.label_4.setText(QCoreApplication.translate("ConversionDialog", u"P2", None))
        self.label_15.setText(QCoreApplication.translate("ConversionDialog", u"P6", None))
        self.label_13.setText(QCoreApplication.translate("ConversionDialog", u"P4", None))
        self.label_5.setText(QCoreApplication.translate("ConversionDialog", u"P3", None))
        self.label_14.setText(QCoreApplication.translate("ConversionDialog", u"P5", None))
        self.label_2.setText(QCoreApplication.translate("ConversionDialog", u"Y = (P1 * X^2 + P2 * X +P3) / (P4 * X^2 + P5 * X + P6)", None))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_2), QCoreApplication.translate("ConversionDialog", u"Rational", None))
        self.vtt_default_btn.setText(QCoreApplication.translate("ConversionDialog", u"Default conversion", None))
        self.insert_btn.setText(QCoreApplication.translate("ConversionDialog", u"Insert", None))
        self.label_16.setText(QCoreApplication.translate("ConversionDialog", u"Default", None))
        self.label.setText(QCoreApplication.translate("ConversionDialog", u"X -> Y (str/float) ", None))
        self.reset_btn.setText(QCoreApplication.translate("ConversionDialog", u"Reset", None))
        self.vtt_default_mode.setItemText(0, QCoreApplication.translate("ConversionDialog", u"Text", None))
        self.vtt_default_mode.setItemText(1, QCoreApplication.translate("ConversionDialog", u"Conversion", None))

        self.tabs.setTabText(self.tabs.indexOf(self.tab_3), QCoreApplication.translate("ConversionDialog", u"Value to Text/Conversion", None))
        self.label_12.setText(QCoreApplication.translate("ConversionDialog", u"[ .. X .. )  -> Y (str/float) ", None))
        self.vrtt_default_btn.setText(QCoreApplication.translate("ConversionDialog", u"Default conversion", None))
        self.insert_vrtt_btn.setText(QCoreApplication.translate("ConversionDialog", u"Insert", None))
        self.reset_vrtt_btn.setText(QCoreApplication.translate("ConversionDialog", u"Reset", None))
        self.label_17.setText(QCoreApplication.translate("ConversionDialog", u"Default", None))
        self.vrtt_default_mode.setItemText(0, QCoreApplication.translate("ConversionDialog", u"Text", None))
        self.vrtt_default_mode.setItemText(1, QCoreApplication.translate("ConversionDialog", u"Conversion", None))

        self.tabs.setTabText(self.tabs.indexOf(self.tab_4), QCoreApplication.translate("ConversionDialog", u"Value range to Text/Conversion", None))
        self.label_18.setText(QCoreApplication.translate("ConversionDialog", u"X -> X", None))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_5), QCoreApplication.translate("ConversionDialog", u"1:1", None))
    # retranslateUi



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'xy.ui'
##
## Created by: Qt User Interface Compiler version 6.7.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QDoubleSpinBox, QGridLayout, QGroupBox,
    QHBoxLayout, QLabel, QLineEdit, QPushButton,
    QSizePolicy, QSlider, QVBoxLayout, QWidget)
from . import resource_rc

class Ui_XYDisplay(object):
    def setupUi(self, XYDisplay):
        if not XYDisplay.objectName():
            XYDisplay.setObjectName(u"XYDisplay")
        XYDisplay.resize(919, 487)
        self.verticalLayout = QVBoxLayout(XYDisplay)
        self.verticalLayout.setSpacing(1)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(1, 1, 1, 1)
        self.plot_layout = QHBoxLayout()
        self.plot_layout.setSpacing(1)
        self.plot_layout.setObjectName(u"plot_layout")

        self.verticalLayout.addLayout(self.plot_layout)

        self.groupBox = QGroupBox(XYDisplay)
        self.groupBox.setObjectName(u"groupBox")
        self.horizontalLayout = QHBoxLayout(self.groupBox)
        self.horizontalLayout.setSpacing(1)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(1, 1, 1, 1)
        self.timestamp = QDoubleSpinBox(self.groupBox)
        self.timestamp.setObjectName(u"timestamp")
        self.timestamp.setDecimals(9)
        self.timestamp.setMinimum(-999999999999.000000000000000)
        self.timestamp.setMaximum(9999999.000000000000000)

        self.horizontalLayout.addWidget(self.timestamp)

        self.min_t = QLabel(self.groupBox)
        self.min_t.setObjectName(u"min_t")

        self.horizontalLayout.addWidget(self.min_t)

        self.timestamp_slider = QSlider(self.groupBox)
        self.timestamp_slider.setObjectName(u"timestamp_slider")
        self.timestamp_slider.setMaximum(99999)
        self.timestamp_slider.setOrientation(Qt.Orientation.Horizontal)
        self.timestamp_slider.setTickInterval(1)

        self.horizontalLayout.addWidget(self.timestamp_slider)

        self.max_t = QLabel(self.groupBox)
        self.max_t.setObjectName(u"max_t")

        self.horizontalLayout.addWidget(self.max_t)

        self.horizontalLayout.setStretch(2, 1)

        self.verticalLayout.addWidget(self.groupBox)

        self.groupBox_2 = QGroupBox(XYDisplay)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout = QGridLayout(self.groupBox_2)
        self.gridLayout.setSpacing(1)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(1, 1, 1, 1)
        self.label = QLabel(self.groupBox_2)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.x_channel_edit = QLineEdit(self.groupBox_2)
        self.x_channel_edit.setObjectName(u"x_channel_edit")

        self.gridLayout.addWidget(self.x_channel_edit, 0, 1, 1, 1)

        self.x_search_btn = QPushButton(self.groupBox_2)
        self.x_search_btn.setObjectName(u"x_search_btn")
        icon = QIcon()
        icon.addFile(u":/search.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.x_search_btn.setIcon(icon)

        self.gridLayout.addWidget(self.x_search_btn, 0, 2, 1, 1)

        self.label_2 = QLabel(self.groupBox_2)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)

        self.y_channel_edit = QLineEdit(self.groupBox_2)
        self.y_channel_edit.setObjectName(u"y_channel_edit")

        self.gridLayout.addWidget(self.y_channel_edit, 1, 1, 1, 1)

        self.y_search_btn = QPushButton(self.groupBox_2)
        self.y_search_btn.setObjectName(u"y_search_btn")
        self.y_search_btn.setIcon(icon)

        self.gridLayout.addWidget(self.y_search_btn, 1, 2, 1, 1)

        self.gridLayout.setColumnStretch(1, 1)

        self.verticalLayout.addWidget(self.groupBox_2)

        self.verticalLayout.setStretch(0, 1)

        self.retranslateUi(XYDisplay)

        QMetaObject.connectSlotsByName(XYDisplay)
    # setupUi

    def retranslateUi(self, XYDisplay):
        XYDisplay.setWindowTitle(QCoreApplication.translate("XYDisplay", u"Form", None))
        self.groupBox.setTitle(QCoreApplication.translate("XYDisplay", u"Time stamp", None))
        self.timestamp.setSuffix(QCoreApplication.translate("XYDisplay", u"s", None))
        self.min_t.setText("")
        self.max_t.setText("")
        self.groupBox_2.setTitle(QCoreApplication.translate("XYDisplay", u"Channels", None))
        self.label.setText(QCoreApplication.translate("XYDisplay", u"X channel", None))
        self.x_search_btn.setText("")
        self.label_2.setText(QCoreApplication.translate("XYDisplay", u"Y channel", None))
        self.y_search_btn.setText("")
    # retranslateUi



from collections import defaultdict
import contextlib
from importlib.metadata import distribution, PackageNotFoundError
import re

from packaging.requirements import Requirement
from PySide6.QtCore import QSize
from PySide6.QtGui import QGuiApplication, QIcon
from PySide6.QtWidgets import (
    QDialog,
    QPushButton,
    QTreeWidget,
    QTreeWidgetItem,
    QVBoxLayout,
)


class DependenciesDlg(QDialog):
    def __init__(self, package_name: str, is_root_package: bool = True) -> None:
        """Create a dialog to list all dependencies for `package_name`."""

        super().__init__()

        # Variables
        self._package_name = package_name
        self._is_root_package = is_root_package

        # Widgets
        self._tree = QTreeWidget()
        self._copy_btn = QPushButton("Copy installed dependencies to clipboard")

        # Setup widgets
        self._setup_widgets()

        # Setup layout
        self._setup_layout()

        # Signals and slots
        self._connect_signals()

    def _setup_widgets(self) -> None:
        self.setWindowTitle(f"Dependency Overview for {self._package_name}")

        self.setMinimumWidth(700)
        self.setMinimumHeight(500)

        icon = QIcon()
        icon.addFile(":/asammdf.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.setWindowIcon(icon)

        headers = ["package", "required", "installed", "summary", "url"]
        self._tree.setHeaderLabels(headers)
        self._populate_tree(self._package_name)
        self._tree.expandAll()
        self._tree.resizeColumnToContents(0)
        self._tree.resizeColumnToContents(1)
        self._tree.resizeColumnToContents(2)

        # enable copy button for root package only
        self._copy_btn.setVisible(self._is_root_package)

    def _setup_layout(self) -> None:
        vbox = QVBoxLayout()
        self.setLayout(vbox)

        vbox.addWidget(self._tree)
        vbox.addWidget(self._copy_btn)

    def _connect_signals(self) -> None:
        self._tree.itemDoubleClicked.connect(self._on_item_double_clicked)
        self._copy_btn.clicked.connect(self._on_copy_button_clicked)

    def _populate_tree(self, package_name: str) -> None:
        package_dist = distribution(package_name)
        requires = package_dist.requires
        if requires is None:
            return

        root_nodes: dict[str, QTreeWidgetItem] = {}

        def get_root_node(name: str | None = None) -> QTreeWidgetItem:
            if name is None:
                name = "mandatory"

            if name in root_nodes:
                return root_nodes[name]

            new_root_node = QTreeWidgetItem([name])
            root_nodes[name] = new_root_node

            font = new_root_node.font(0)
            font.setBold(True)
            new_root_node.setFont(0, font)

            self._tree.invisibleRootItem().addChild(new_root_node)
            return new_root_node

        for group, requirements in grouped_dependencies(package_name).items():
            for req_string in requirements:
                req = Requirement(req_string)
                parent_node = get_root_node(group)

                item = QTreeWidgetItem()
                item.setText(0, req.name)
                item.setText(1, str(req.specifier))

                with contextlib.suppress(PackageNotFoundError):
                    dist = distribution(req.name)
                    item.setText(2, str(dist.version))
                    item.setText(3, dist.metadata["Summary"])
                    item.setText(4, dist.metadata["Home-Page"])

                parent_node.addChild(item)

    def _on_item_double_clicked(self, item: QTreeWidgetItem, column: int) -> None:
        if column != 0:
            return
        if item.parent() is self._tree.invisibleRootItem():
            return

        package_name = item.text(0)
        DependenciesDlg.show_dependencies(package_name, is_root_package=False)

    def _on_copy_button_clicked(self) -> None:
        """Create a list of all dependencies and their versions and write it to clipboard."""
        lines: list[str] = []
        dependencies = find_all_dependencies(self._package_name)
        max_name_length = max(len(name) for name in dependencies)

        header = f"{'Package':<{max_name_length}} Version"
        lines.append(header)
        lines.append("-" * len(header))
        for name in sorted(dependencies):
            version = distribution(name).version
            lines.append(f"{name:<{max_name_length}} {version}")

        # write to clipboard
        QGuiApplication.clipboard().setText("\n".join(lines))

    @staticmethod
    def show_dependencies(package_name: str, is_root_package: bool = True) -> None:
        dlg = DependenciesDlg(package_name, is_root_package)
        dlg.exec()


def grouped_dependencies(package_name: str) -> dict[str, list[str]]:
    """Retrieve a dictionary grouping the dependencies of a given package into mandatory and optional categories.

    This function fetches the dependencies of the specified package and categorizes them into groups, such as
    'mandatory' or any optional feature groups specified by `extra` markers.

    :param package_name:
        The name of the package to analyze.
    :return:
        A dictionary where keys are group names (e.g., 'mandatory', 'extra_feature')
        and values are lists of package names corresponding to those groups.
    """
    dependencies: defaultdict[str, list[str]] = defaultdict(list)
    package_dist = distribution(package_name)

    if requires := package_dist.requires:
        for req_string in requires:
            req = Requirement(req_string)

            group = "mandatory"
            if match := re.search(r"extra\s*==\s*['\"](?P<extra>\S+)['\"]", str(req.marker)):
                group = match["extra"]

            dependencies[group].append(req_string)
    return dependencies


def find_all_dependencies(package_name: str) -> set[str]:
    """Recursively find all dependencies of a given package, including transitive dependencies.

    This function determines all dependencies of the specified package, following any transitive dependencies
    (i.e., dependencies of dependencies) and returning a complete set of package names.

    :param package_name:
        The name of the package to analyze.
    :return:
        A set of all dependencies for the package, including transitive dependencies.
    """

    def _flatten_groups(grouped_deps: dict[str, list[str]]) -> set[str]:
        _dep_set = set()
        for group, requirements in grouped_deps.items():
            _dep_set |= {Requirement(req_string).name for req_string in requirements}
        return _dep_set

    dep_set: set[str] = {package_name}
    todo = _flatten_groups(grouped_dependencies(package_name))
    while todo:
        req_name = todo.pop()
        if req_name in dep_set:
            continue
        try:
            todo |= _flatten_groups(grouped_dependencies(req_name))
        except PackageNotFoundError:
            continue
        dep_set.add(req_name)
    return dep_set



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'channel_info_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.7.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGroupBox, QHBoxLayout, QLabel,
    QScrollArea, QSizePolicy, QSpacerItem, QVBoxLayout,
    QWidget)

class Ui_ChannelInfo(object):
    def setupUi(self, ChannelInfo):
        if not ChannelInfo.objectName():
            ChannelInfo.setObjectName(u"ChannelInfo")
        ChannelInfo.resize(926, 558)
        self.horizontalLayout = QHBoxLayout(ChannelInfo)
        self.horizontalLayout.setSpacing(1)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(1, 1, 1, 1)
        self.groupBox = QGroupBox(ChannelInfo)
        self.groupBox.setObjectName(u"groupBox")
        font = QFont()
        font.setFamilies([u"Consolas"])
        font.setPointSize(8)
        self.groupBox.setFont(font)
        self.groupBox.setFlat(False)
        self.verticalLayout_4 = QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setSpacing(1)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(1, 1, 1, 1)
        self.scrollArea = QScrollArea(self.groupBox)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 295, 529))
        self.verticalLayout_3 = QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(1, 1, 1, 1)
        self.channel_label = QLabel(self.scrollAreaWidgetContents)
        self.channel_label.setObjectName(u"channel_label")
        self.channel_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.verticalLayout_3.addWidget(self.channel_label)

        self.verticalSpacer_3 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_3.addItem(self.verticalSpacer_3)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.verticalLayout_4.addWidget(self.scrollArea)


        self.horizontalLayout.addWidget(self.groupBox)

        self.groupBox_2 = QGroupBox(ChannelInfo)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setFont(font)
        self.verticalLayout_6 = QVBoxLayout(self.groupBox_2)
        self.verticalLayout_6.setSpacing(1)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.verticalLayout_6.setContentsMargins(1, 1, 1, 1)
        self.scrollArea_2 = QScrollArea(self.groupBox_2)
        self.scrollArea_2.setObjectName(u"scrollArea_2")
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollAreaWidgetContents_2 = QWidget()
        self.scrollAreaWidgetContents_2.setObjectName(u"scrollAreaWidgetContents_2")
        self.scrollAreaWidgetContents_2.setGeometry(QRect(0, 0, 294, 529))
        self.verticalLayout_2 = QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.verticalLayout_2.setSpacing(1)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.conversion_label = QLabel(self.scrollAreaWidgetContents_2)
        self.conversion_label.setObjectName(u"conversion_label")
        self.conversion_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.verticalLayout_2.addWidget(self.conversion_label)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.scrollArea_2.setWidget(self.scrollAreaWidgetContents_2)

        self.verticalLayout_6.addWidget(self.scrollArea_2)


        self.horizontalLayout.addWidget(self.groupBox_2)

        self.groupBox_3 = QGroupBox(ChannelInfo)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.groupBox_3.setFont(font)
        self.verticalLayout_5 = QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setSpacing(1)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(1, 1, 1, 1)
        self.scrollArea_3 = QScrollArea(self.groupBox_3)
        self.scrollArea_3.setObjectName(u"scrollArea_3")
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollAreaWidgetContents_3 = QWidget()
        self.scrollAreaWidgetContents_3.setObjectName(u"scrollAreaWidgetContents_3")
        self.scrollAreaWidgetContents_3.setGeometry(QRect(0, 0, 295, 529))
        self.verticalLayout = QVBoxLayout(self.scrollAreaWidgetContents_3)
        self.verticalLayout.setSpacing(1)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(1, 1, 1, 1)
        self.source_label = QLabel(self.scrollAreaWidgetContents_3)
        self.source_label.setObjectName(u"source_label")
        self.source_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.verticalLayout.addWidget(self.source_label)

        self.verticalSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer_2)

        self.scrollArea_3.setWidget(self.scrollAreaWidgetContents_3)

        self.verticalLayout_5.addWidget(self.scrollArea_3)


        self.horizontalLayout.addWidget(self.groupBox_3)


        self.retranslateUi(ChannelInfo)

        QMetaObject.connectSlotsByName(ChannelInfo)
    # setupUi

    def retranslateUi(self, ChannelInfo):
        ChannelInfo.setWindowTitle(QCoreApplication.translate("ChannelInfo", u"Form", None))
        self.groupBox.setTitle(QCoreApplication.translate("ChannelInfo", u"Channel", None))
        self.channel_label.setText(QCoreApplication.translate("ChannelInfo", u"Channel metadata", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("ChannelInfo", u"Conversion", None))
        self.conversion_label.setText(QCoreApplication.translate("ChannelInfo", u"No conversion", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("ChannelInfo", u"Source", None))
        self.source_label.setText(QCoreApplication.translate("ChannelInfo", u"No source", None))
    # retranslateUi



"""ASAM MDF version 2 file format module"""

from os import PathLike

from typing_extensions import Unpack

from .mdf_v3 import Kwargs, MDF3
from .utils import FileLike, MdfException, validate_version_argument
from .v2_v3_constants import Version2

__all__ = ["MDF2"]


# MDF versions 2 and 3 share the same implementation
class MDF2(MDF3):
    """shared implementation for mdf version 2 and 3"""

    def __init__(
        self,
        name: str | PathLike[str] | FileLike | None = None,
        version: Version2 = "2.14",
        **kwargs: Unpack[Kwargs],
    ) -> None:
        version = validate_version_argument(version, hint=2)

        if not kwargs.get("__internal__", False):
            raise MdfException("Always use the MDF class; do not use the class MDF2 directly")

        super().__init__(name, version, **kwargs)


if __name__ == "__main__":
    pass
